#!/usr/bin/env python

from subprocess import check_call, CalledProcessError
import click
from os.path import join, isdir, isfile
import os
import sys


@click.command(
    help='Migrate limeviews and configs stored in filesystem to database')
@click.argument('application')
def migrate(application):
    migrate_views(application)
    migrate_configs(application)


def migrate_views(application):
    views_dir = join('C:\\', 'ProgramData', 'Lundalogik', 'LIME Pro Server',
                     'applications', application, 'lime_view')

    if not isdir(views_dir):
        print('Application {} does not seem to have any views to migrate.'
              .format(application))
        return

    view_files = [f for f in os.listdir(views_dir)]

    if len(view_files) < 1:
        print('Application {} does not seem to have any views to migrate.'
              .format(application))
        return

    for f in view_files:
        key = 'limeview.{}'.format(f)
        view_file = join(views_dir, f)
        cmd = ['limefu', 'data', 'set', '-a', application, '-k', key]

        try:
            print('Migrating limeview for {}...'.format(f))
            with open(view_file, 'r') as vf:
                check_call(cmd, stdin=vf)

        except CalledProcessError as e:
            print('Migrate limeviews failed!\nError:{}'.format(e),
                  file=sys.stderr)
            return

    print('Migrated {} limeviews for application {} successfully!'
          .format(len(view_files), application))

    print('Removing limeview files...')
    for f in view_files:
        view_file = join(views_dir, f)
        os.remove(view_file)
    os.rmdir(views_dir)


def migrate_configs(application):
    application_dir = join(
        'C:\\', 'ProgramData', 'Lundalogik', 'LIME Pro Server', 'applications',
        application)

    if not isdir(application_dir):
        print('Application {} does not seem to have any configs to migrate.'
              .format(application))
        return

    configs = [('webclient_add.yml', 'webclient.add')]

    migrated = []

    for filename, key in configs:
        config_file = join(application_dir, filename)

        if not isfile(config_file):
            continue

        migrated.append(config_file)

        print('Found config {}, migrating...'.format(filename))

        cmd = ['limefu', 'data', 'set', '-a', application, '-k', key]

        try:
            with open(config_file, 'r') as vf:
                check_call(cmd, stdin=vf)

        except CalledProcessError as e:
            print('Migrate {} failed!\nError:{}'.format(filename, e),
                  file=sys.stderr)
            return

    print('Migrated {} configs for application {} successfully!'
          .format(len(migrated), application))

    print('Removing config files...')
    for filename in migrated:
        os.remove(filename)


if __name__ == '__main__':
    migrate()
